<?php
/**
 * @version   $Id$
 * @author    RocketTheme http://www.rockettheme.com
 * @copyright Copyright (C) 2007 - 2013 RocketTheme, LLC
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 only
 */

abstract class RokSprocket_Provider_AbstarctWordpressBasedProvider extends RokSprocket_Provider
{


	/**
	 * @return RokSprocket_ItemCollection
	 */
	public function getItems()
	{
		global $wpdb;

		/** @var $filer_processor RokSprocket_Provider_AbstractWordpressPlatformFilter */
		$filer_processor = $this->getFilterProcessor();
		$filer_processor->setWidgetId($this->module_id);
        $filer_processor->setDisplayedIds($this->displayed_ids);
		$manualsort = ($this->params->get($this->params->get('provider', 'joomla') . '_sort', 'automatic') == 'manual') ? true : false;
		$filer_processor->setManualSort($manualsort);
		$filer_processor->process($this->filters, $this->sort_filters, $this->showUnpublished);

		$query = $filer_processor->getQuery();

		$raw_results = $wpdb->get_results($query, OBJECT_K);

		if ($raw_results === false) {
			throw new RokSprocket_Exception($wpdb->last_error);
		}

		$converted = $this->convertRawToItems($raw_results);
		$this->mapPerItemData($converted);
		return $converted;
	}

	function filter_where($where = '')
	{
		return $where;
	}

	/**
	 * @param array $data
	 *
	 * @return RokSprocket_ItemCollection
	 */
	protected function convertRawToItems(array $data)
	{
		$collection = new RokSprocket_ItemCollection();
		$dborder    = 0;
		foreach ($data as $raw_item) {
			$item                              = $this->convertRawToItem($raw_item, $dborder);
			$collection[$item->getArticleId()] = $item;
			$dborder++;
		}
		return $collection;
	}

	/**
	 * @abstract
	 *
	 * @param     $raw_item
	 * @param int $dborder
	 */
	abstract protected function convertRawToItem($raw_item, $dborder = 0);

	/**
	 * @param RokSprocket_ItemCollection $items
	 *
	 * @throws RokSprocket_Exception
	 */
	protected function getModuleItemSettings(RokSprocket_ItemCollection &$items)
	{
		global $wpdb;
		//TODO move this to be a platform independent fucntion
		$item_ids = array_keys($items);
		$query    = '';
		$query .= 'SELECT rsi.provider_id as id, rsi.order as order, rsi.params as params';
		$query .= ' FROM #__roksprocket_items as rsi';
		$query .= ' WHERE ' . sprintf('rsi.widget_id = %d', $this->module_id);
		$query .= ' AND ' . sprintf('rsi.provider = %s', '"' . $this->provider_name . '"');
		$query .= ' AND ' . sprintf('rsi.provider_id in (%s)', implode(',', $item_ids));
		$query .= ' ORDER BY rsi.order';
		$query        = str_replace('#__', $wpdb->prefix, $query); //add wp db prefix
		$item_results = $wpdb->get_results($query, OBJECT_K);
		if ($item_results === false) {
			throw new RokSprocket_Exception($wpdb->last_error);
		}
		foreach ($item_results as $item_id => $item) {
			if (isset($items[$item_id])) {

			}
		}
	}

	/**
	 * @param RokSprocket_ItemCollection $items
	 *
	 * @throws RokSprocket_Exception
	 */
	protected function mapPerItemData(RokSprocket_ItemCollection &$items)
	{
		global $wpdb;
		$query = 'SELECT i.provider_id as id, i.order, i.params';
		$query .= ' FROM #__roksprocket_items as i';
		$query .= ' WHERE i.widget_id = "' . $this->module_id . '"';
		$query .= ' AND i.provider = "' . $this->provider_name . '"';
		$query          = str_replace('#__', $wpdb->prefix, $query); //add wp db prefix
		$sprocket_items = $wpdb->get_results($query, OBJECT_K);
		if ($sprocket_items === false) {
			throw new RokSprocket_Exception($wpdb->last_error);
		}

		/** @var $items RokSprocket_Item[] */
		foreach ($items as $item_id => &$item) {
			list($provider, $id) = explode('-', $item_id);
			if (array_key_exists($id, $sprocket_items)) {
				$items[$item_id]->setOrder((int)$sprocket_items[$id]->order);
				if (!is_null($sprocket_items[$id]->params)) {
					$items[$item_id]->setParams(RokCommon_Utils_ArrayHelper::fromObject(RokCommon_JSON::decode($sprocket_items[$id]->params)));
				} else {
					$items[$item_id]->setParams(array());

				}
			}
		}
	}

	/**
	 * @param $id
	 *
	 * @return \RokSprocket_Item
	 */
	public function getArticlePreview($id)
	{
		$ret = $this->getArticleInfo($id);
		$ret->setText($this->_cleanPreview($ret->getText()));
		return $ret;
	}

	/**
	 * @param      $id
	 *
	 * @param bool $raw return the raw object not the RokSprocket_Item
	 *
	 * @return stdClass|RokSprocket_Item
	 * @throws RokSprocket_Exception
	 */
	public function getArticleInfo($id, $raw = false)
	{
		global $wpdb;
		/** @var $filer_processor RokCommon_Filter_IProcessor */
		$filer_processor = $this->getFilterProcessor();
		$filer_processor->process(array('id' => array($id)), array(), true);
		$query = $filer_processor->getQuery();
		$ret   = $wpdb->get_results($query, OBJECT);
		$ret   = $ret[0];
		if ($ret === false) {
			throw new RokSprocket_Exception($wpdb->last_error);
		}

		if ($raw) {
			$ret->preview = $this->_cleanPreview($ret->introtext);
			$ret->editUrl = $this->getArticleEditUrl($id);
			return $ret;
		} else {
			$item          = $this->convertRawToItem($ret);
			$item->editUrl = $this->getArticleEditUrl($id);
			$item->preview = $this->_cleanPreview($item->getText());
			return $item;
		}
	}

	/**
	 * @abstract
	 *
	 * @param $id
	 */
	abstract protected function getArticleEditUrl($id);

	/**
	 * @param $content
	 *
	 * @return mixed
	 */
	protected function _cleanPreview($content)
	{
		//Replace src links
		$base = get_bloginfo('wpurl');

		$regex   = '#href="index.php\?([^"]*)#m';
		$content = preg_replace_callback($regex, array('self', '_route'), $content);

		$protocols = '[a-zA-Z0-9]+:'; //To check for all unknown protocals (a protocol must contain at least one alpahnumeric fillowed by :
		$regex     = '#(src|href)="(?!/|' . $protocols . '|\#|\')([^"]*)"#m';
		$content   = preg_replace($regex, "$1=\"$base\$2\" target=\"_blank\"", $content);

		$regex   = '#(onclick="window.open\(\')(?!/|' . $protocols . '|\#)([^/]+[^\']*?\')#m';
		$content = preg_replace($regex, '$1' . $base . '$2', $content);

		// ONMOUSEOVER / ONMOUSEOUT
		$regex   = '#(onmouseover|onmouseout)="this.src=([\']+)(?!/|' . $protocols . '|\#|\')([^"]+)"#m';
		$content = preg_replace($regex, '$1="this.src=$2' . $base . '$3$4"', $content);

		// Background image
		$regex   = '#style\s*=\s*[\'\"](.*):\s*url\s*\([\'\"]?(?!/|' . $protocols . '|\#)([^\)\'\"]+)[\'\"]?\)#m';
		$content = preg_replace($regex, 'style="$1: url(\'' . $base . '$2$3\')', $content);

		$content = strip_shortcodes($content);

		return $content;
	}

	/**
	 * @param $matches
	 *
	 * @return string
	 */
	protected function _route(&$matches)
	{
		$original = $matches[0];
		$url      = $matches[1];
		$url      = str_replace('&amp;', '&', $url);
		$route    = get_bloginfo('wpurl') . $url;

		return 'target="_blank" href="' . $route;
	}
}
